/*
 * Copyright 2014, Broadcom Corporation
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior
 * written permission of Broadcom Corporation.
 */

/** @file
 *
 * UART UDP Bridge Application
 *
 * This application snippet demonstrates uart to UDP bridge.
 *
 * Features demonstrated
 *  - Wi-Fi softAP mode
 *  - DHCP server
 *  - UDP transmit
 *  - UART Receive
 *
 * Application Instructions
 *   1. Connect a PC terminal to the serial port of the WICED Eval board,
 *      then build and download the application as described in the WICED
 *      Quick Start Guide
 *   2. Connect PC to Wifi Access point UART SOFT AP
 *   3. listen to udp port 8000
 *   4. Open serial terminal
 *   5. When 10 bytess are send through serial terminal whole 10bytes will be received at UDP Port
 *
 */

#include "wiced.h"

/******************************************************
 *                      Macros
 ******************************************************/

#define UDP_MAX_DATA_LENGTH        300
#define UDP_RX_TIMEOUT              1
#define UDP_TX_INTERVAL             1
#define UDP_RX_INTERVAL             1
#define UDP_TARGET_PORT          8000

#define UART_THREAD_PRIORITY          (WICED_APPLICATION_PRIORITY)
#define UART_THREAD_STACKSIZE         (4096)

#define UDP_TARGET_IS_BROADCAST

#define RX_BUFFER_SIZE             64

#ifdef UDP_TARGET_IS_BROADCAST
    #define UDP_TARGET_IP MAKE_IPV4_ADDRESS(192,168,0,255)
#else
    #define UDP_TARGET_IP MAKE_IPV4_ADDRESS(192,168,0,2)
#endif

/******************************************************
 *                    Constants
 ******************************************************/

/******************************************************
 *                   Enumerations
 ******************************************************/

/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Function Declarations
 ******************************************************/

static wiced_result_t tx_udp_packet();
static void uart_thread_main();

/******************************************************
 *               Variables Definitions
 ******************************************************/

static const wiced_ip_setting_t device_init_ip_settings =
{
    INITIALISER_IPV4_ADDRESS( .ip_address, MAKE_IPV4_ADDRESS(192,168,  0,  1) ),
    INITIALISER_IPV4_ADDRESS( .netmask,    MAKE_IPV4_ADDRESS(255,255,255,  0) ),
    INITIALISER_IPV4_ADDRESS( .gateway,    MAKE_IPV4_ADDRESS(192,168,  0,  1) ),
};

static wiced_udp_socket_t  udp_socket;

  wiced_packet_t*          packet;
  char*                    udp_data;
  uint16_t                 available_data_length;
  const wiced_ip_address_t INITIALISER_IPV4_ADDRESS( target_ip_addr, UDP_TARGET_IP );

wiced_uart_config_t uart_config =
{
    .baud_rate    = 115200,
    .data_width   = DATA_WIDTH_8BIT,
    .parity       = NO_PARITY,
    .stop_bits    = STOP_BITS_1,
    .flow_control = FLOW_CONTROL_DISABLED,
};

wiced_ring_buffer_t rx_buffer;
uint8_t             rx_data[RX_BUFFER_SIZE];

unsigned char Buffer[50],Bufcount=0;

static wiced_thread_t      uart_thread;
/******************************************************
 *               Function Definitions
 ******************************************************/

void application_start(void)
{
    /* Initialise the device and WICED framework */
    wiced_init( );

    /* Initialise ring buffer */
     ring_buffer_init(&rx_buffer, rx_data, RX_BUFFER_SIZE );
     /* Initialise UART. A ring buffer is used to hold received characters */
     wiced_uart_init( WICED_UART_1, &uart_config, &rx_buffer );

    /* Bring up the softAP and network interface */
    wiced_network_up( WICED_AP_INTERFACE, WICED_USE_INTERNAL_DHCP_SERVER, &device_init_ip_settings );

    /* Create UDP socket */
    if (wiced_udp_create_socket(&udp_socket, UDP_TARGET_PORT, WICED_AP_INTERFACE) != WICED_SUCCESS)
    {
      //  WPRINT_APP_INFO(("UDP socket creation failed\r\n"));
    }

    wiced_rtos_create_thread(&uart_thread,UART_THREAD_PRIORITY,"UART SOFT AP", uart_thread_main,UART_THREAD_STACKSIZE,NULL);

}

void uart_thread_main()
{

    char read;

    while ( wiced_uart_receive_bytes( WICED_UART_1, &read, 1, WICED_NEVER_TIMEOUT ) == WICED_SUCCESS )
    {

            Buffer[Bufcount++]=(unsigned char)read;
            if(Bufcount >=10)
            {
                Buffer[Bufcount++]=0;
                tx_udp_packet();
                Bufcount=0;
            }
    }
    WICED_END_OF_THREAD(uart_thread);
}
/*
 * Sends a UDP packet
 */
wiced_result_t tx_udp_packet()
{

    /* Create the UDP packet */
    if (wiced_packet_create_udp(&udp_socket, UDP_MAX_DATA_LENGTH, &packet, (uint8_t**)&udp_data, &available_data_length) != WICED_SUCCESS)
    {
        WPRINT_APP_INFO(("UDP tx packet creation failed\r\n"));
        return WICED_ERROR;
    }

    /* Write packet number into the UDP packet data */
    //sprintf(udp_data, "%d", (int)tx_count++);
    strcpy(udp_data,(char *)Buffer);

    /* Set the end of the data portion */
    wiced_packet_set_data_end(packet, (uint8_t*)udp_data + Bufcount);

    /* Send the UDP packet */
    if (wiced_udp_send(&udp_socket, &target_ip_addr, UDP_TARGET_PORT, packet) != WICED_SUCCESS)
    {
    //    WPRINT_APP_INFO(("UDP packet send failed\r\n"));
        wiced_packet_delete(packet);  /* Delete packet, since the send failed */
        return WICED_ERROR;
    }

    return WICED_SUCCESS;

}


